<?php

namespace Drupal\dkan_group;

use Drupal\Component\Uuid\UuidInterface;
use Drupal\group\Entity\Group;
use Drupal\group\Entity\GroupRole;
use Drupal\group\GroupMembershipLoader;
use Drupal\json_form_widget\StringHelper;
use Drupal\metastore\MetastoreService;
use Symfony\Component\DependencyInjection\ContainerInterface;

class WidgetRouter extends \Drupal\json_form_widget\WidgetRouter {


  protected $groupHandler;

  protected $groupMembershipLoader;

  public function __construct(UuidInterface $uuid, StringHelper $string_helper, MetastoreService $metastore, DkanGroupHandler $dkan_group_handler, GroupMembershipLoader $group_membership_loader)
  {
    parent::__construct($uuid, $string_helper, $metastore);
    $this->groupHandler = $dkan_group_handler;
    $this->groupMembershipLoader = $group_membership_loader;
  }

  public static function create(ContainerInterface $container)
  {
    return new static(
      $container->get('uuid'),
      $container->get('json_form.string_helper'),
      $container->get('dkan.metastore.service'),
      $container->get('dkan_group.group_handler'),
      $container->get('group.membership_loader')
    );
  }

  /**
   * @{inheritDocs}
   *
   * skip publishers the user is no member of
   */
  public function getOptionsFromMetastore(mixed $source, mixed $titleProperty = FALSE)
  {
    $options = [];
    $metastore_items = $this->metastore->getAll($source->metastoreSchema);
    foreach ($metastore_items as $item) {
      $item = json_decode($item);
      $title = $this->metastoreOptionTitle($item, $titleProperty);
      $value = $this->metastoreOptionValue($item, $source, $titleProperty);
      // We skip publishers the user is no member of
      if ($source->metastoreSchema === 'publisher') {
        $group = $this->groupHandler->getGroupFromPublisher($item->identifier);
        if ($group instanceof Group) {
          if ($group->getMember(\Drupal::currentUser()) === FALSE && (!GroupRole::load('publisher-admin_out')->getGlobalRole() || !in_array(GroupRole::load('publisher-admin_out')->getGlobalRole()->id(), \Drupal::currentUser()->getRoles()))) {
            continue;
          }
        }
      }
      $options[$value] = $title;
    }
    return $options;
  }

  /**
   * @{inheritDocs}
   *
   * control autocreate property based on schema UI property groupRoleCheck.
   *   add group roles that are allowed to use autocreate on this element.
   * control required property based on schema UI property groupRoleRequired
   *   these roles require a value for this element. can be used to only
   *   require a value for some roles (i. e. to not allow group members
   *   to add content without a value)
   */
  public function getDropdownElement(mixed $element, mixed $spec, mixed $titleProperty = FALSE)
  {
    $element = parent::getDropdownElement($element, $spec, $titleProperty);
    if (isset($spec->groupRoleCheck)) {
      $auto_create = FALSE;
      foreach ($spec->groupRoleCheck as $role_id) {
        $role = GroupRole::load($role_id);
        if ($role->getGlobalRole()) {
          $auto_create = in_array($role->getGlobalRole()->id(), \Drupal::currentUser()->getRoles());
        }
        else {
          $auto_create = !empty($this->groupMembershipLoader->loadByUser(\Drupal::currentUser(), $role_id));
        }
        if ($auto_create) {
          break;
        }
      }
      if ($element['#type'] === 'select2') {
        if ($auto_create === FALSE) {
          $element['#select2_tags'] = 'false';
        }
      }
      else {
        $element['#autocreate'] = $auto_create;
      }
    }
    if (isset($spec->groupRoleRequire)) {
      $require = $element['#required'] ?? FALSE;
      foreach ($spec->groupRoleRequire as $role_id) {
        if (in_array($role_id, \Drupal::currentUser()->getRoles())) {
          $require = TRUE;
          break;
        }
      }
      $element['#required'] = $require;
    }
    return $element;
  }

}
